//obsługa wyświetlacza LCD
#include <LiquidCrystal.h>
//obsługa pamięci EEPROM
#include <EEPROM.h>

//port PB
#define pb0  8
#define pb1  9
#define pb2  10
#define pb3  11
#define pb4  12
#define pb5  13

//port C
#define pc0  23
#define pc1  24
#define pc2  25
#define pc3  26
#define pc4  27
#define pc5  28

//port D
#define pd0  0
#define pd1  1
#define pd2  2
#define pd3  3
#define pd4  4
#define pd5  5
#define pd6  6
#define pd7  7

/*************************************
 zakresy
 *************************************/
//liczba impulsów - bezwzględna, większa od 0
//dolna granica
#define  DOLNE_IMPULSY  1
//górna granica
#define  GORNE_IMPULSY  10000
//domyślna liczba impulsow
#define  DOMYSLNE_IMPULSY  100  //100 impulsów
//odstęp pomiędzy impulsami - liczba bezwzględna, większa od 0
//jednostką sa milisekundy; uwaga: nie może być mniej niż 50!
//najkrótszy odstęp
#define  DOLNY_OKRES  100
//najdłuższy odstęp
#define  GORNY_OKRES  10000
//domyślny odstęp
#define  DOMYSLNY_OKRES  1000  //1 sekunda
//czas trwania impulsu - liczba bezwzględna, większa od 0
//jednostką sa mikrosekundy; uwaga: nie może być mniej niż 3!
//najkrótszy czas trwania
#define  DOLNY_CZAS_TRWANIA  100
//najdłuższy czas trwania
#define  GORNY_CZAS_TRWANIA  10000
//domyślny czas trwania
#define  DOMYSLNY_CZAS_TRWANIA  1000 //1 sekunda


//definicje wyprowadzeń dla diod LED
#define  red  11
#define  green  10
#define  blue  12

//zmienna z nastawą
int liczba_impulsow, okres, czas_trwania_impulsow;
//stan impulsatora
int tmp, y_markera = 0;
//bit ze stanem przycisku i ze stanem enkodera
boolean imp_przycisk, w_lewo, w_prawo, poprzedni;

//dołączenie wyświetlacza
LiquidCrystal lcd(pb0, pb1, pd4, pd5, pd6, pd7);

//zapis parametrów w pamieci eeprom, komórka o adresie 0 zawiera
//znacznik, jeśli znacznik jest równy 0x55, to eeprom zawiera ważne
//dane, inaczej są nadawane wartości domyślne
void eeprom_zapis_nastaw()
{
  EEPROM.write(6, lowByte(czas_trwania_impulsow));
  EEPROM.write(5, highByte(czas_trwania_impulsow));
  EEPROM.write(4, lowByte(liczba_impulsow));
  EEPROM.write(3, highByte(liczba_impulsow));
  EEPROM.write(2, lowByte(okres));
  EEPROM.write(1, highByte(okres));
  EEPROM.write(0, 0x55);
}
  
//odczyt parametrów
void eeprom_odczyt_nastaw()
{
  if (EEPROM.read(0) != 0x55)
  {
    okres = DOMYSLNY_OKRES;
    liczba_impulsow = DOMYSLNE_IMPULSY;
    czas_trwania_impulsow = DOMYSLNY_CZAS_TRWANIA;
    eeprom_zapis_nastaw();
  }
  else
  {
    okres = EEPROM.read(1);
    okres = okres << 8;
    okres = okres + EEPROM.read(2);
    liczba_impulsow = EEPROM.read(3);
    liczba_impulsow = liczba_impulsow << 8;
    liczba_impulsow = liczba_impulsow + EEPROM.read(4);
    czas_trwania_impulsow = EEPROM.read(5);
    czas_trwania_impulsow = czas_trwania_impulsow << 8;
    czas_trwania_impulsow = czas_trwania_impulsow + EEPROM.read(6);
  }
}

/**************************************************
 * zaświecanie i gaszenie diody RGB
 **************************************************/
void led_off()
{
  digitalWrite(red, LOW);
  digitalWrite(green, LOW);
  digitalWrite(blue, LOW);
}

void led_red()
{
  led_off();
  digitalWrite(red, HIGH);
}

void led_green()
{
  led_off();
  digitalWrite(green, HIGH);
}

void led_blue()
{
  led_off();
  digitalWrite(blue, HIGH);
}

/**************************************************
 obsługa enkodera
 **************************************************/
void odczyt_impulsatora()
{
  boolean imp_pin1, imp_pin2, biezacy;
  unsigned int i = 0;
//odczyt stanu przycisku enkodera
  imp_przycisk = w_prawo = w_lewo = LOW;
  imp_pin1 = digitalRead(pd0);
  if(imp_pin1 == LOW)
  {
    delay(50);
    imp_pin1 = digitalRead(pd0);
    if(imp_pin1 == LOW) imp_przycisk = HIGH;
    else imp_przycisk = LOW;
  }
//odczyt kierunku obrotu
  imp_pin1 = digitalRead(pd1);
  if(imp_pin1 == LOW) biezacy = HIGH; else biezacy = LOW;
  if(poprzedni != biezacy)
  {
    poprzedni = biezacy;
    if(imp_pin1 == LOW)
    {
      imp_pin2 = digitalRead(pd2);
      if(imp_pin2 == LOW)
      {
        w_lewo = HIGH;
        w_prawo = LOW;
      }
      else
      {
        w_lewo = LOW;
        w_prawo = HIGH;
      }
    }
  }
}

//oczekiwanie na zwolnienie ośki
void czekaj_na_przycisk()
{
  while (imp_przycisk == HIGH) odczyt_impulsatora();
}


/**************************************************
 obsługa generatora
 **************************************************/
//generowanie impulsów na wyjściu przekaźnikowym
void impuls(int czas_trwania)
{
  digitalWrite(pc0, LOW);
  delayMicroseconds(czas_trwania);
  digitalWrite(pc0, HIGH);
}

//pętla główna generatora
void start_generatora()
{
  int i, n, ODST, t;

/*obliczenia:
  - n =liczba obrotów pętli,
  - ODST =odstęp pomiędzy impulsami,
  - t =czas trwania impulsu. */
  n = liczba_impulsow;        //bezwymiarowa
  t = czas_trwania_impulsow;  //wymiar us
  ODST = okres - (float(t)/1000);   //wymiar ms
  
//zielona dioda LED
  led_green();
  lcd.setCursor(0, 0);
  lcd.print("POZOSTA-");
  lcd.setCursor(0, 1);
  lcd.print("LO:     ");
//pętla głowna generatora  
  while (n > 0)
  {
//wyświetlenie numeru przebiegu
    lcd.setCursor(3, 1);
    lcd.print(n);
    lcd.print(" ");
//generowanie kolejnych impulsów o czasie trwania t-mikrosekund
    impuls(t);
//oczekiwanie o skorygowany czas ODST-milisekund
    for(i=0; i<ODST; i++) delayMicroseconds(1000);
    n--;
  }
//zaświecenie czerwonej diody LED
  led_red();
  czekaj_na_przycisk();
}


/**************************************************
 obsługa nastaw parametrów
 **************************************************/
//wyświetlenie menu nastaw parametrów
void wyswietlenie_menu_nastaw(int numer_linii)
{
  if (tmp != numer_linii)
  {
    switch (numer_linii)
    {
      case 0:
        lcd.setCursor(0, 0);
        lcd.print(">IMPULSY");
        lcd.setCursor(0, 1);
        lcd.print("        ");
        lcd.setCursor(1, 1);
        lcd.print(liczba_impulsow);
        break;
      case 1:
        lcd.setCursor(0, 0);
        lcd.print(">ODSTEP ");
        lcd.setCursor(0, 1);
        lcd.print("        ");
        lcd.setCursor(1, 1);
        lcd.print(okres);
        lcd.print("ms");
        break;
      case 2:
        lcd.setCursor(0, 0);
        lcd.print(">CZAS TR");
        lcd.setCursor(0, 1);
        lcd.print("        ");
        lcd.setCursor(1, 1);
        lcd.print(czas_trwania_impulsow);
        lcd.print("ms");
        break;
      case 3:
        lcd.setCursor(0, 0);
        lcd.print(">OK     ");
        lcd.setCursor(0, 1);
        lcd.print("        ");
        break;
      case 4:
        lcd.setCursor(0, 0);
        lcd.print(">Anuluj ");
        lcd.setCursor(0, 1);
        lcd.print("        ");
        break;
      default:
        break;
    }
    tmp = numer_linii;
  }
}

//wyświetlenie znaku zachęty
void znak_zachety_dol()
{
//czyszczenie znaczku zachęty na górze
  lcd.setCursor(0, 0);
  lcd.print(" ");
//wyświetlenie znaku zachęty na dole
  lcd.setCursor(0, 1);
  lcd.print(">");
//zaświecenie niebieskiej diody LED
  led_blue();
}

void znak_zachety_gora()
{
//czyszczenie znaczku zachęty na górze
  lcd.setCursor(0, 1);
  lcd.print(" ");
//wyświetlenie znaku zachęty na dole
  lcd.setCursor(0, 0);
  lcd.print(">");
//zaświecenie czerwonej diody LED
  led_red();
}

//ustawianie odstępu pomiędzy impulsami
int ustaw_czas_trwania(int lpoczatkowa)
{
  int tmp = lpoczatkowa;

  znak_zachety_dol();
//oczekiwanie na zwolnienie ośki impulsatora
  czekaj_na_przycisk();
  do
  {
    odczyt_impulsatora();
    if (w_lewo == HIGH && tmp > DOLNY_CZAS_TRWANIA)
    {
      tmp = tmp-10;
      lcd.setCursor(0, 1);
      lcd.print(">      ");
      lcd.setCursor(7, 1);
      lcd.print(" ");
      lcd.setCursor(1, 1);
      lcd.print(tmp);
      lcd.print("ms");
    }
    else
      if (w_prawo == HIGH && tmp < GORNY_CZAS_TRWANIA)
      {
        tmp = tmp+10;
        lcd.setCursor(0, 1);
        lcd.print(">      ");
        lcd.setCursor(7, 1);
        lcd.print(" ");
        lcd.setCursor(1, 1);
        lcd.print(tmp);
        lcd.print("ms");
      }
  } while (imp_przycisk == LOW);
//oczekiwanie na puszczenie ośki
  czekaj_na_przycisk();
  znak_zachety_gora();
  return(tmp);
}

//ustawianie liczby impulsów
int ustaw_okres(int lpoczatkowa)
{
  int tmp = lpoczatkowa;

  znak_zachety_dol();
//oczekiwanie na zwolnienie ośki impulsatora
  czekaj_na_przycisk();
  do
  {
    odczyt_impulsatora();
    if (w_lewo == HIGH && tmp > DOLNY_OKRES)
    {
      tmp--;
      lcd.setCursor(0, 1);
      lcd.print(">      ");
      lcd.setCursor(1, 1);
      lcd.print(tmp);
      lcd.print("ms");
    }
    else
      if (w_prawo == HIGH && tmp < GORNY_OKRES)
      {
        tmp++;
        lcd.setCursor(0, 1);
        lcd.print(">      ");
        lcd.setCursor(1, 1);
        lcd.print(tmp);
        lcd.print("ms");
      }
  } while (imp_przycisk == LOW);
//oczekiwanie na puszczenie ośki
  czekaj_na_przycisk();
  znak_zachety_gora();
  return(tmp);
}

//ustawienie liczby generowanych impulsow 
int ustaw_liczbe_impulsow(int lpoczatkowa)
{
  int tmp = lpoczatkowa;
  
  znak_zachety_dol();
//oczekiwanie na zwolnienie ośki impulsatora
  czekaj_na_przycisk();
  do
  {
    odczyt_impulsatora();
    if (w_lewo == HIGH && tmp > DOLNE_IMPULSY)
    {
      tmp--;
      lcd.setCursor(0, 1);
      lcd.print(">      ");
      lcd.setCursor(1, 1);
      lcd.print(tmp);
    }
    else
      if (w_prawo == HIGH && tmp < GORNE_IMPULSY)
      {
        tmp++;
        lcd.setCursor(0, 1);
        lcd.print(">      ");
        lcd.setCursor(1, 1);
        lcd.print(tmp);
      }
  } while (imp_przycisk == LOW);
//oczekiwanie na puszczenie ośki
  czekaj_na_przycisk();
  znak_zachety_gora();
  return(tmp);
}

//ustawianie liczby impulsów i czasu ich trwania
void ustawianie_parametrow()
{
  int poz = 0, tliczba = liczba_impulsow, tokres = okres, tczas = czas_trwania_impulsow;
  
  czekaj_na_przycisk();
  do
  {
//to menu ma 5 pozycji, rolowane na ekranie
    odczyt_impulsatora();
    if (w_prawo == HIGH)
    {
      poz--;
      if (poz < 0) poz = 4;
    }
    else
      if (w_lewo == HIGH)
      {
        poz++;
        if (poz > 4) poz = 0;
      }
      else
        if (imp_przycisk == HIGH)
        {
          switch (poz)
          {
            case 0:
              liczba_impulsow = ustaw_liczbe_impulsow(liczba_impulsow);
              break;
            case 1:
              okres = ustaw_okres(okres);
              break;
            case 2:
              czas_trwania_impulsow = ustaw_czas_trwania(czas_trwania_impulsow);
              break;
            case 4:
//przepisanie zmiennych tymczasowych do stałych
//(odtworzenie wartości sprzed edycji)
              okres = tokres;
              liczba_impulsow = tliczba;
              czas_trwania_impulsow = tczas;
              break;
            case 3:
//zapamiętanie nastaw w pamięci nieulotnej
              eeprom_zapis_nastaw();
            default:
              break;
          }
        }
    wyswietlenie_menu_nastaw(poz);
  } while (imp_przycisk == LOW);
}


/**************************************************
 akcja po wyborze pozycji z menu
 **************************************************/
//wywołanie odpowieniej akcji
void akcja(int numer_linii)
{
  czekaj_na_przycisk();
  switch(numer_linii)
  {
    case 0:
      start_generatora();
      break;
    case 1:
      lcd.clear();
//konieczne, aby tmp <> numer_linii, bo tylko wtedy
//zadziała wyświetlanie menu nastaw
      tmp = 99;
      wyswietlenie_menu_nastaw(0);
      ustawianie_parametrow();
      czekaj_na_przycisk();
      break;
  }
}

//kasowanie starego markera
void kasuj_marker()
{
  lcd.setCursor(0, y_markera);
  lcd.print(" ");
}

//wywietlenie nowego markera
void ustaw_marker()
{
  lcd.setCursor(0, y_markera);
  lcd.print(">");
}


/**************************************************
 inicjalizacja mikrokontrolera
 **************************************************/
void setup()
{
//wejścia enkodera
  pinMode(pd0, INPUT_PULLUP);
  pinMode(pd1, INPUT);
  pinMode(pd2, INPUT);
//sterowanie diodą LED
  pinMode(red, OUTPUT);
  pinMode(green, OUTPUT);
  pinMode(blue, OUTPUT);
//wyjście generatora
  pinMode(pc0, OUTPUT);
//zerowanie wyjścia generatora
  digitalWrite(pc0, HIGH);
//wyświetlenie menu głównego
  lcd.begin(8, 2);
//zaświecenie czerwonej diody LED
  led_red();
//odczyt nastaw z pamięci nieulotnej
  eeprom_odczyt_nastaw();
}


/**************************************************
 główna pętla programu
 **************************************************/
void menu_glowne(int poz)
{
  switch (poz)
  {
    case 0:
      lcd.setCursor(0, 0);
      lcd.print(">URUCHOM");
      lcd.setCursor(0, 1);
      lcd.print(" GENER. ");
      break;      
    case 1:
      lcd.setCursor(0, 0);
      lcd.print(">PARAME-");
      lcd.setCursor(0, 1);
      lcd.print(" TRY    ");
      break;      
  }
}

void loop()
{
//wyświetlenie menu głównego
  menu_glowne(y_markera);
//odczyt stanu impulsatora
  odczyt_impulsatora();
//ruch markera w dół
  if (w_lewo == HIGH)
  {
    y_markera--;
    if (y_markera < 0) y_markera = 1;
  } else
//ruch markera w górę
    if (w_prawo == HIGH)
    {
      y_markera++;
      if (y_markera > 1) y_markera = 0;
    } else
//wywołanie akcji po naciśnięciu impulsatora
//akcja zależna od położenia w menu
      if (imp_przycisk == HIGH)
      {
        akcja(y_markera);
//przywrócenie ekranu startowego
        y_markera = 0;
      }
}
