/*
Przykład obsługi komponentów jakie zawiera moduł z wyświetlaczem LED dla Arduino UNO.
Program zawiera przykład konfiguracji i obsługi:
- wyświetlacza LED
- zegara RTC
- 2 przycisków
- brzęczyka PIEZO
- czujnika temperatury DS18B20
- czujnika światła
*/


#include "TimerOne.h"  //biblioteka obslugi timera
#include <Wire.h> //biblioteka i2c
#include <PCF8583.h>  //biblioteka RTC
#include <OneWire.h> //biblioteka 1Wire

OneWire  ds(A3);        // konfiguracja linii 1Wire
byte present = 0;      //definicje zmiennych uzywanych w programie
byte data[12];
byte addr[8];
int wart_analog;  
int HighByte, LowByte, SignBit, temp, Fract, TReading, Tc_100;
PCF8583 p (0xA0);       //konfiguracja adresu ukladu RTC
const int Buzzer =  13;   //przypisanie aliasow linia portow
const int DP =  12; 
const int SW1 =  A1; 
const int SW2 =  A2;
const int sens_sw =  A0; 
int groundPins[8] = {0, 1, 2, 3, 4, 5, 6, 7};   //linie portow wierszy LED
int digitPins[4] = { 8, 9, 10, 11}; //linie portow kolumn LED
int wys=0;
int digit[4]; //tablica znakow dla wyswietlacza LED
int kr[4];   //tablica przechowujaca zak kropki
byte temp_sec;
byte wsk_sek=1;
int number[13][7] = {    //tablica znakow dla LED
  {0,0,0,0,0,0,1}, //zero
  {1,0,0,1,1,1,1}, //jeden
  {0,0,1,0,0,1,0}, //dwa
  {0,0,0,0,1,1,0}, //trzy
  {1,0,0,1,1,0,0}, //cztery
  {0,1,0,0,1,0,0}, //piec
  {0,1,0,0,0,0,0}, //szesc
  {0,0,0,1,1,1,1}, //siedem
  {0,0,0,0,0,0,0}, //osiem
  {0,0,0,0,1,0,0}, //dziewiec
  {1,1,1,1,1,1,1}, //wylaczenie LED 
  {0,0,1,1,1,0,0}, //znak stopnia
  {0,1,1,0,0,0,1}  //znak C
};

void setup()        //procedura konfiguracyjna
{ 
  Timer1.initialize(100);      //inicjalizacja timer 1
  Timer1.attachInterrupt(int_wys);   //uruchomienie przerwania od timer 1 w procedurze int_wys
  for(int i=0; i < 8; i++)      //konfiguracja linii jako wyjscia obslugujacych wiersze LED
  {
    pinMode(groundPins[i], OUTPUT);
    digitalWrite(groundPins[i], HIGH);
  } 
  pinMode(Buzzer, OUTPUT);     //konfiguracja PIEZO
  digitalWrite(Buzzer, HIGH);
  pinMode(DP, OUTPUT);        //konfiguracja dwukropka wyswietlacza LED
  digitalWrite(DP, HIGH);

  for(int i=0; i < 4; i++)   //konfiguracja kolumn wyswietlacza LED
  {
    pinMode(digitPins[i], OUTPUT);
    digitalWrite(digitPins[i], HIGH);
  } 
  p.hour = 14;  //wartosci poczatkowe dla ukladu RTC
  p.minute = 30;
  p.second = 0;
  p.year = 2011;
  p.month = 4;
  p.day = 1;
  p.set_time();
  pinMode(SW1, INPUT);        //konfiguracja linii do ktorych sa dolaczono przyciski jako wejscia
  pinMode(SW2, INPUT);
  digitalWrite(SW1, HIGH);   //dolaczenie do linii do ktorych sa dolaczone przyciski rezystorow podciagajacych co wymusi na nich domyslnie stan wysoki
  digitalWrite(SW2, HIGH);
  analogReference(DEFAULT);   //konfiguracja przetwornika A/C

} 

void int_wys() {      //funkcja przerwania w ktorej obslugiwany jest wyswietlacz LED
  for(int i=0; i < 4; i++)    //petla for wykonywana dla kazdego z wyswietlaczy
  {
    digitalWrite(digitPins[i], HIGH);  //wylaczenie wyswietlacza
  }   
  for(int g=0; g < 7; g++)     //zapisanie do wyswietlacza znaku zapisanego w tablicy digit
  {
    digitalWrite(groundPins[g], number[digit[wys]][g]);
  };
  digitalWrite(groundPins[7], !(kr[wys]));  //zapisanie stanu kropki
  digitalWrite(digitPins[wys], LOW);    //wlaczenie obslugiwanego wyswietlacza
  wys++;       //zwiekszenie wartosci o 1 zmiennej wys
  if (wys>4) wys=0;     //jesli obslugiwany czwarty wyswietlacz to przejscie do obslugi pierwszego wyswietlacza
};

void loop()     //petla glowna programu
{
  digitalWrite(DP, LOW);   //wlaczenie calego wyswietlacza LED 
  kr[0]=HIGH;
  kr[1]=HIGH;
  kr[2]=HIGH;
  kr[3]=HIGH;
  for(int k=0; k < 10; k++)   //wyswietlenie na LED znakow od 0 do 9
  {
    digit[0] = k;
    digit[1] = k;
    digit[2] = k;
    digit[3] = k;
    delay(200);          //opoznienie 200ms
  };

  digitalWrite(DP, HIGH);  //instrukcje wylaczajace wszystkie elementy wyswietlacza LED
  kr[0]=LOW;
  kr[1]=LOW;
  kr[2]=LOW;
  kr[3]=LOW;
  digit[0] = 10;
  digit[1] = 10;
  digit[2] = 10;
  digit[3] = 10;
  delay(2000);
  while(1)      //petla programu
  {
    p.get_time();  //pobranie z RTC danych o czasie i dacie
    digit[3]=  p.minute%10;     //wyswietlenie minut
    digit[2] = p.minute / 10;
    digit[1]=  p.hour%10;     //wyswietlenie godzin
    digit[0] = p.hour / 10;

    if (temp_sec!=p.second)    //instrukcje migania dwukropka wyswietlacza co 1 sekunde
    {
      temp_sec=p.second;   //zapisanie wartosci sekund
      if (wsk_sek==1)   //jesli zmienna ma wartosc 1 to 
        digitalWrite(DP, HIGH);  //wylaczenie dwukropka
      else          //w przeciwnym razie
        digitalWrite(DP, LOW);  //wlaczenie dwukropka
      wsk_sek = !wsk_sek;  //zmianna stanu zmiennej
    } 

    if (digitalRead(SW1) == LOW) {         //Sprawdzenie czy nacisniety przycisk S1
      digit[0] = 10;  //instrukcje wylaczajace wyswietlacz LED
      digit[1] = 10;
      digit[2] = 10;
      digit[3] = 10;
      digitalWrite(DP, HIGH);   
      while(digitalRead(SW1) == LOW)   //jesli przycisk nadal przycisniety to
      {
        getTemp();              //pobranie wartosci temperatury 
        digit[1]=  temp%10;     //wyswietlenie wartosci temperatury
        digit[0] = temp / 10;
        digit[2] = 11;          //wyswietlenie znaku stopnia
        digit[3] = 12;          //wyswietlenie znaku C
        delay(500);            //opoznienie 500 ms
      }
    }
    if (digitalRead(SW2) == LOW) {         //Sprawdzenie czy nacisniety przycisk S2
      digit[0] = 10;            //instrukcje wylaczajace LED
      digit[1] = 10;
      digit[2] = 10;
      digit[3] = 10;
      digitalWrite(DP, HIGH);
      while(digitalRead(SW2) == LOW)  //jesli przycisk S2 nadal nacisniety to
      {
        digit[3]=  p.day%10; //wyswietlenie dnia miesiaca
        digit[2] = p.day / 10;
        digit[1]=  p.month%10; //wyswietlenie miesiaca
        digit[0] = p.month / 10;
        kr[1]=HIGH;          //wlaczenie kropki drugiego wyswietlacza
        delay(500);     //opoznienie 500 ms
      }
      kr[1]=LOW;   //wylaczenie kropki    
    }

    if ((wart_analog = analogRead(A0))<80)  //jesli wartosc odczytana z czujnika swiatla jest ponizej wartosci 80 to
      digitalWrite(Buzzer, LOW);   //wlaczenie brzeczyka
    else  //w przeciwnym razie 
      digitalWrite(Buzzer, HIGH);   //wylaczenie brzeczyka
  }  
} 

void getTemp() {    //procedura odczytu temperatury z czujnika 1Wire DS18B20
  int foo, bar, i;   //definicje zmiennych
  ds.reset();  //zerowanie ukladu 1Wire
  ds.write(0xCC,1);   //komendy startujace pomier temperatury
  ds.write(0x44,1);

  ds.reset();   //instrukcje odczytu temperatury
  ds.write(0xCC,1);
  ds.write(0xBE);

  for ( i = 0; i < 9; i++) {
    data[i] = ds.read();
  }
  LowByte = data[0];  //instrukcje obliczajace temperature ktorej wynik jest zapisywany
  HighByte = data[1];  //do zmiennej temp
  TReading = (HighByte << 8) + LowByte;
  SignBit = TReading & 0x8000;  //obliczenie znaku temperatury
  if (SignBit) {
    TReading = -TReading;
  }
  Tc_100 = (6 * TReading) + TReading / 4;    
  temp = Tc_100 / 100;          
  Fract = Tc_100 % 100;
  if (Fract > 49) {
    if (SignBit) {
      --temp;
    } 
    else {
      ++temp;
    }
  }
} 




