/*
Przykład obsługi modułu AVTDuino SD
W pierwszej kolejnosci inicjowana jest karta SD wraz z wyswietleniem jej typu i pojemnosci.
Zakladany jest plik test.txt w ktorym zapisywany jest przykladowy tekst.
Nastepnie zawartosc utworzonego pliku jest odczytywana po czym plik zostaje usuniety z karty SD.
Wszystkie funkcje wykonywane na karcie SD sa wysylane w postaci komunikatow przez port RS232 ktore mozna odczytac w terminalu.
 */
#include <SD.h>  //Biblioteka obsługi karty SD

Sd2Card card;  //zmienne pomocnicze do obslugi karty
SdVolume volume;
SdFile root;  
File myFile;   //zmienna potrzebna dla utworzenia pliku

const int chipSelect = 8;  //konfiguracja pinu aktywujacego karte SD    

void setup()    //procedura konfiguracyjna
{
  Serial.begin(9600);    //ustawienie predkosci interfejsu RS232
  Serial.print("\nInicjalizacja karty SD...");  //komunikat
  SD.begin(8);    //inicjacja SD
  pinMode(10, OUTPUT);     
  if (!card.init(SPI_HALF_SPEED, chipSelect)) {   //jesli inicjacja nie przebiegla prawidlowo to
    Serial.println("Brak karty");  //wyswietlenie komunikatu
    while(1);  //pusta petla
  }
}


void loop(void) {  //procedura glowna programu
  Serial.print("\nTyp karty:: ");  //komunikat
  switch(card.type()) {   //sprawdzenie typu karty
  case SD_CARD_TYPE_SD1:     //jest typ SD1
    Serial.println("SD1");  //komunikat
    break;
  case SD_CARD_TYPE_SD2:     //jesli typ SD2
    Serial.println("SD2");  //komunikat
    break; 
  case SD_CARD_TYPE_SDHC:  //jesli typ SDHC
    Serial.println("SDHC");  //komunikat
    break;
  default:    //w przeciwnym razie
    Serial.println("Nieznany");   //typ karty nieznany
  }

  if (!volume.init(card)) {  //sprawdzenie typu partycji karty
    Serial.println("Typ partycji nieznany");  //komunikat
    while(1);  //pusta petla
  }

  uint32_t volumesize;  //zmienna
  Serial.print("\nTyp partycji: ");  //wyswietlenie odczytanego typu partycji
  Serial.println(volume.fatType(), DEC);
  Serial.println(); //pusta linia

  volumesize = volume.blocksPerCluster();   //obliczenie pojemnosci karty SD
  volumesize *= volume.clusterCount();       
  volumesize *= 512;                        
  volumesize /= 1024;
  volumesize /= 1024;
  Serial.print("Pojemnosc: (Mb): ");  //wyswietlenie pojemnosci
  Serial.println(volumesize);

  Serial.println("Otwarcie pliku test.txt");  //komunikat
  myFile = SD.open("test.txt", FILE_WRITE);  //ustworzenie pliku test.txt

  if (myFile) {    //jesli plik zostal utworzony to zapis do niego przykladowego komunikatu
    Serial.println("Zapis danych do pliku...");
    myFile.println("Przykladowy tekst dla modulu AVTDuino SD");
    myFile.close();  //zamkniecie pliku
  } 
  else { //w przeciwnym razie wyswietlenie
    Serial.println("Blad otwarcia pliku test.txt");  //komunikatu
  }
  
  Serial.println("Wyswietlenie zawartosci karty:");  //komunikat
  root.openRoot(volume);   //wyswietlenie zawartosci karty
  root.ls(LS_R | LS_DATE | LS_SIZE);
  
  Serial.println("Wyswietlenie zawartosci pliku test.txt"); //komunikat
  myFile = SD.open("test.txt");  //otwarcie pliku
  if (myFile) {   //jesli plik zostal otwarty to
    Serial.println("Zawartosc pliku test.txt:");
    while (myFile.available()) {   //odczyt jego zawartosci
      Serial.write(myFile.read()); //odczyt
    }
    myFile.close();  //zamkniecie pliku
  } 
  else {   //w przeciwnym razie wyswietlenie
    Serial.println("Blad otwarcia pliku test.txt");  //komunikatu
  }

  Serial.println("Kasowanie pliku test.txt...");  //komunikat
  SD.remove("test.txt");   //skazowanie pliku test.txt
  if (SD.exists("test.txt")){   //sprawdzenie czy plik zostal usuniety
    Serial.println("Blad usuniecia pliku test.txt.");  //komunikat
  }
  else {  //w przeciwnym razie
    Serial.println("Plik test.txt skasowaniy.");  //komunikat
  }
  while(1);  //pusta petla
}


