//------------------------------------------------------
#include <Servo.h>   //dołączenie biblioteki do obsługi serw
Servo servox;       //tworzymy obiekt kontrolujacy serwo 1
Servo servoy;       //tworzymy obiekt kontrolujacy serwo 2

//------------------------------------------------------
int servox_pin = 8;  //sterowanie serwa 1 na wyprowadzeniu 8
int servoy_pin = 9;  //sterowanie serwa 2 na wyprowadzeniu 9
int joyx_pin = 0;    //sygnał z osi x joysticka na wyprowadzeniu 0
int joyy_pin = 1;    //sygnał z osi y joysticka na wyprowadzeniu 1

//------------------------------------------------------
int analog_x;        //wychylenie osi x
int analog_y;        //wychylenie osi y
int degree_x;        //położenie serwa osi x
int degree_y;        //położenie serwa osi y

//------------------------------------------------------
int led1_pin = 6;
int led2_pin = 5;
int button1_pin = 3;
int button2_pin = 4;

//------------------------------------------------------
int pos_go;         //funkcja replay - gdy 1 to odtwarza położenia
int pos_step;       //numer położenia 
int pos_x[4];       //tablica położeń na płaszczyźnie poziomej
int pos_y[4];       //tablica położeń na płaszczyźnie pionowej


//----------------------------------------------------------
void setup()
{
servox.attach(servox_pin);  //serwo x przypisane do wyprowadzenia 8
servoy.attach(servoy_pin);  //serwo y przypisane do wyprowadzenia 9

degree_x = 90;    //położenie początkowe serva osi x  
degree_y = 90;    //położenie początkowe serva osi y 

pinMode(led1_pin, OUTPUT);
pinMode(led2_pin, OUTPUT);
pinMode(button1_pin, INPUT_PULLUP);
pinMode(button2_pin, INPUT_PULLUP);
}

//----------------------------------------------------------
void loop()
{
int temp;  //zmienna pomocnicza

//odczyt położenia joystika,
//odczyt wartosci analogowych
analog_x = analogRead(joyx_pin);     
analog_y = analogRead(joyy_pin);

//przeskalowanie wartosci analogowej
//na wartość kąta odchylenia serwomechanizmów
//oraz odwrócenie ruchu - teraz podąża za joystickiem
temp = map(analog_x, 0, 1023, 180, 0);

//gdy funkcja replay to odtwazra zapisane położenia
if (pos_go > 0) temp = pos_x[pos_step];
  
//powolne dochodzenie do zadanej wartości kąta x
//o 1 jeden stopień przy każdym przejściu pętli 
if (degree_x < temp) degree_x++;
if (degree_x > temp) degree_x--;
  
//przeskalowanie wartosci analogowej
//na wartość kąta odchylenia serwomechanizmów
temp = map(analog_y, 0, 1023, 0, 180);  

//gdy funkcja replay to odtwazra zapisane położenia
if (pos_go > 0) temp = pos_y[pos_step];

//powolne dochodzenie do zadanej wartości kata y
//o 1 jeden stopień przy każdym przejściu pętli 
if (degree_y < temp) degree_y++;
if (degree_y > temp) degree_y--;

//ustawienie serwomechanizmów
servox.write(degree_x);
servoy.write(degree_y);

//przerwa, ustala szybkość ruchu serw
delay(30);

//sprawdzenie przycisków
temp = ButtonCheck();

//przycisk 1 - zapamietywanie położenia
if (temp == 1)
{
  PositionSet(pos_step, degree_x, degree_y);
  pos_step++;
  if (pos_step >= 4) pos_step = 0;
}

//przycisk 2 - włączanie funkcji replay
if (temp == 2)
{
  if (pos_go == 0)
  pos_go = 1;
  else
  pos_go = 0;
}

//steruje diodami obrazując stan urządzenia
LedState(pos_step, pos_go);

//automatycznie przełacza zapisane pozycje
// po określonej liczbie cykli
PositionNext(150);
}

//------------------------------------------------------
//sprawdza stan przycisków, 
//zwraca wartość tylko w momencie przyciśnięcia
int ButtonCheck(void)
{
//zmienne statyczne,
//zapamietują ostatni stan przycisków
static int button1_state;    
static int button2_state;
int result = 0;

if (digitalRead(button1_pin) == LOW)
{
  //jeśli przycisk był zwolniony to zwraca wartość
  if (button1_state == 0) result = 1;
  button1_state = 1;  
}
else
  button1_state = 0; 
  
if (digitalRead(button2_pin) == LOW)
{
  //jeśli przycisk był zwolniony to zwraca wartość
  if (button2_state == 0) result = 2;
  button2_state = 1;  
}
else
  button2_state = 0;

return result;
}

//------------------------------------------------------
//ustawia diody led w zależnosści od stanu urządzenia
void LedState(int pos, int go)
{
if (pos > 0)  //w czasie programowania pozycji świeci LED1
  digitalWrite(led1_pin, LOW);
else
  digitalWrite(led1_pin, HIGH);
  
if (go > 0)   //w czasie odtwarzania sekwencji świeci LED2
  digitalWrite(led2_pin, LOW);
else
  digitalWrite(led2_pin, HIGH);  
}

//------------------------------------------------------
//zapamietuje położenie x i y w tablicach
void PositionSet(int pos_num, int x, int y)
{ 
if (pos_step >= 4) pos_step = 0;
pos_x[pos_num] = degree_x;
pos_y[pos_num] = degree_y;  
}

//------------------------------------------------------
//przełącza numer sekwencji po okreśonym czasie trwan
void PositionNext(int how_long)
{
static int counter;
if (pos_go > 0)
{
  counter++;
  if (counter >= how_long)  //następna pozycja
  {
  counter = 0;
  pos_step++;
  //jeśli to ostatnia to zaczyna od początku  
  if (pos_step >= 4) pos_step = 0;  
  } 
}
else
  counter = 0;
}

